# #!/usr/bin/env python
# from sallron.util import settings
# from sallron.db import mongo
# from walrus import Database
# from redis import Redis
# from time import sleep
# import datetime
# import schedule
# import logging
# import os
# import sys

# _REDIS_HOST = '127.0.0.1' # standard

# class RedisCache():
#     def __init__(self):

#         self.cache_logger = logging.getLogger('cache')
#         self.cache = Database().cache()
#         self.cache = Database().cache()

#         self.connection_retries = 0
#         self.max_connection_retries = 3

#         # redis-server check and handling
#         if not(self._check_connection()):
#             self.cache_logger.warning("Redis-server not running!")
#             try:
#                 self.cache_logger.info("Starting redis-server on background.")
#                 self._start_redis()
#             except:
#                 # self.cache_logger.warning("Redis-server not installed!")
#                 # self._install_redis()
#                 self.cache_logger.warning("Redis-server could not be started!")
#                 sys.exit()

#     def schedule_new_customers_checking(self, mongo_client, interface, interface_name, fn):
#         """
#         Utility function to schedule checking of customers

#         Args:
#             mongo_client (MongoDB class instance): MongoDB class instance defined in ```e_raptor/db/mongo.py```
#             interface (str): Interface that is running at the moment.
#             fn (function object): Function to be scheduled in case a new customer was added.
#         """
#         self.cache.set(f"{interface_name}_customers", mongo_client.databases)

#         # schedule checking new customers
#         schedule.every().day.at("15:00").do(
#             self._check_customers,
#             interface=interface,
#             interface_name=interface_name,
#             fn=fn
#         )

#     def _check_customers(self, interface, interface_name, fn):
#         """
#         Utility function to check whether a new customer was added to MongoDB.
#         In case it was, schedule all functions to the new customer

#         Args:
#             interface (str): Interface that is running at the moment.
#             fn (function object): Function to be scheduled in case a new customer was added.
#         """

#         current_customers = self.cache.get(f"{interface_name}_customers")

#         real_customers = mongo.MongoDB().databases

#         intersection = list(
#             set(current_customers).intersection(real_customers)
#         )

#         if intersection == real_customers:

#             self.cache_logger.info("No new customer added between {} and {}".format(
#                 self.cache.get("lastRun"),
#                 datetime.datetime.now().strftime("%Y-%m-%d %H:%M:%S")
#             ))
        
#         else:

#             self.cache.set(f"{interface_name}_customers", real_customers)

#             for customer in set(real_customers) - set(current_customers):

#                 fn(customer, interface, interface_name)
#                 self.cache_logger.info("New customer added between {} and {}: {}".format(
#                     self.cache.get("lastRun"),
#                     datetime.datetime.now().strftime("%Y-%m-%d %H:%M:%S"),
#                     customer
#                 ))

#             # schedule.run_all() # run_all catastrophe

#         # set date of last time this function has run
#         self.cache.set("lastRun", datetime.datetime.now().strftime("%Y-%m-%d %H:%M:%S"))

#     def _check_connection(self):
#         r = Redis(_REDIS_HOST, socket_connect_timeout=1) # short timeout for the test
#         try:
#             r.ping() 
#             self.cache_logger.info('Connected to Redis @"{}"'.format(_REDIS_HOST))
#             return True
#         except:
#             self.cache_logger.info('Failed to ping Redis, check your connection.')
#             return False

#     def _start_redis(self):
#         if settings.OS == 'UBUNTU':
#             os.popen("service redis-server start --daemonize yes")
#         else:
#             os.popen("redis-server --daemonize yes")
#         sleep(2)
#         if (self._check_connection()):
#             self.cache_logger.info("Redis-server started.")
#             self.connection_retries = 0
#         else:
#             self.connection_retries += 1
#             if self.connection_retries < self.max_connection_retries:
#                 self.cache_logger.info(f"Failed to start redis-server, retrying for the {self.connection_retries} time.")
#                 self._start_redis()
#             else:
#                 self.cache_logger.warning("Max retries reached, check your redis-server installation|config.")

#     def _install_redis(self):
#         """
#         Try to stay away from using this function. Not tested.
#         """
#         os.popen("sudo -s")
#         if settings.OS == 'UBUNTU':
#             os.popen("apt install redis-server")
#         else:
#             os.popen("brew install redis")
#         sleep(10)
#         self.cache_logger.info("Redis-server installed.")



from sallron.util import settings, stats
from discord_logger import DiscordLogger

def send_message(message, level="error", title="Exception raised", description=""):
    """
    Sends a message to a discord channel.

    Args:
        message (string): the message to be sent.
        interface_name (string): The name of the interface
        level (string): The level of the information being sent. Default is "error". Possibilities: error, warn, info, verbose, debug, success.
        title (string): Title of the message.
        description (string): Description of the message.
    """
    webhook_url = settings.DISCORD_WEBHOOK
    metadata = stats.metadata_dict()
    options = {
        "application_name": "Sallron",
        "service_name": f"{settings.INTERFACE_NAME}",
        "service_environment": "Production",
        "default_level": "info",
    }

    logger = DiscordLogger(webhook_url=webhook_url, **options)

    if level == 'error':
        logger.construct(
            title=f"{title}",
            level=f"{level}",
            description=f"{description}",
            error=message,
            metadata=metadata,
        )
    else:
        logger.construct(
            title=f"{title}",
            level=f"{level}",
            description=f"{description}\n{message}",
            metadata=metadata,
        )

    response = logger.send()

    pass


from sallron.eye import configureye, ring_ruler
from sallron.enforcer import eternal_runner
from sallron.extensions import e_raptor
from sallron.util.logger import *
from sallron.util import settings



from sallron.util import settings, logger, discord
import time
import sys
import psutil
import logging
from subprocess import Popen
from os.path import join, dirname, exists
from os import environ

_KILL_TIMEOUT = 3
_PID_FILE = 'sallron.pid'

enforcer_logger = logging.getLogger('enforcer')

def send_message():
    """
    To be implemented.
    The idea here is:
        1. overwrite standard exception flow with custom exception_logger found in util/logger.py
        2. send message with resulting logs on every restart
        3. accompany message with deploy + customer name
    """
    if exists(_PID_FILE):
        enforcer_logger.warning("Sallron had to be restarted, take a look at the error logs...")
        try:
            errorfile = logger.get_logname(settings.LOG_DIR, 'errors')
            with open(errorfile, 'r') as errf:
                fullfile = errf.read()
                last_traceback = fullfile.split('<--->')[-2]
                formatted_traceback = f'```{last_traceback[-1800:]}```'
                discord.send_message(
                    message='Take a look at the last traceback',
                    level='warn',
                    title='Sallron had to be restarted',
                    description=formatted_traceback
                    # description = 'Hello papai'
                    )
        except:
            discord.send_message(
                message='No traceback fetched',
                level='verbose',
                title='Sallron had to be restarted',
                description='Error fetching traceback file'
                )
    else:
        discord.send_message(
                    message=f'Interface: {settings.INTERFACE_NAME}',
                    level='success',
                    title='Sallron starting up',
                    description='Seems like a clean start, no sallron.pid file found.'
                    )
    pass

def write_pid(pid):
    with open(_PID_FILE, 'w') as w:
        w.write(pid)
    enforcer_logger.info(f'pid {pid} written in {_PID_FILE}')

def kill_pid(delete_file=True):
    '''Kill processes.'''
    with open(_PID_FILE, 'r') as rapid:
        pid = rapid.read()
    if delete_file:
        Popen(f'rm {_PID_FILE}', shell=True)
        enforcer_logger.info(f'{_PID_FILE} purged')
    Popen('kill %s' % pid, shell=True)
    enforcer_logger.info(f'pid {pid} killed')

def eternal_runner(filepath, test=False):
    while True:
        try:
            send_message()
            enforcer_logger.info(f'Running {filepath} file')
            p = Popen(f"python3 {filepath}", shell=True)
            if test:
                raise KeyboardInterrupt
            p.wait()
        except (KeyboardInterrupt):
            enforcer_logger.info('Exit signal received, shutting down.')
            time.sleep(_KILL_TIMEOUT)
            kill_pid()
            sys.exit()
            print("That's all folks.")


#!/usr/bin/env python

import logging
from logging.handlers import RotatingFileHandler
from sallron.util import settings, discord
from datetime import datetime
import traceback
import logging
import sys
import os

def get_logname(directory, name, extension="txt"):
    ''' returns logs/master_2020-10-30.txt for instance'''
    if directory.endswith('/'):
        return f"{directory}{name}_{datetime.today().strftime('%Y-%m-%d')}.{extension}"
    else:
        return f"{directory}/{name}_{datetime.today().strftime('%Y-%m-%d')}.{extension}"

def setup_logger(level = logging.INFO):
    '''Prints logger info to terminal'''
    logger = logging.getLogger()
    logger.setLevel(level)
    ch = logging.StreamHandler()
    formatter = logging.Formatter("%(asctime)s - %(name)s - %(levelname)s - %(message)s")
    ch.setFormatter(formatter)
    logger.addHandler(ch)
    return logger

def setup_master_logger(logdir, maxsize, maxbackups=5, level=logging.INFO):
    rfh = logging.handlers.RotatingFileHandler(
        filename=get_logname(logdir, 'master'), 
        mode='a',
        maxBytes=maxsize,
        backupCount=maxbackups,
        encoding=None,
        delay=0
    )
    logging.basicConfig(handlers=[rfh],
                                format='%(asctime)s,%(msecs)d %(name)s %(levelname)s %(message)s',
                                datefmt='%H:%M:%S',
                                level=level)


# Global error_logger so all functions bellow can use it
error_logger = logging.getLogger('errors')

def setup_error_logger(logdir, maxsize):
    '''Basic handler setup for error_logger'''
    log_path = get_logname(logdir, 'errors')

    global error_logger
    error_logger = logging.getLogger('errors')
    try:
        handler = logging.handlers.RotatingFileHandler(log_path, maxBytes=maxsize, backupCount=1) # stream=sys.stdout
    except FileNotFoundError:
        os.mkdir(logdir)
        handler = logging.handlers.RotatingFileHandler(log_path, maxBytes=maxsize, backupCount=1) # stream=sys.stdout
    error_logger.addHandler(handler)

def close_error_logger():
    error_logger.removeHandler(error_logger.handlers[0])
    return

def log_exception(exc_type, exc_value, exc_traceback):
    '''Log unhandled exceptions
    set sys.excepthook = logger.log_exception on main file for it to work'''
    if issubclass(exc_type, KeyboardInterrupt):
        sys.__excepthook__(exc_type, exc_value, exc_traceback)
        return
    message = f'{exc_type}: {exc_value}'
    trace = ''.join(traceback.format_tb(exc_traceback))
    description = f"```\n{trace}\n```"
    discord.send_message(
        message=message,
        description=description)

    error_logger.info(str(datetime.now()))
    error_logger.error("Uncaught exception", exc_info=(exc_type, exc_value, exc_traceback))
    error_logger.info('----------------<--->------------------') # '<--->' used so latest error parsing is easier
    close_error_logger()
    return

def log_error(message):
    '''Log handled errors'''
    discord.send_message(message=message, title='Error ocurred')
    error_logger.info(str(datetime.now()))
    error_logger.error(message)
    error_logger.info('----------------<--->------------------')
    return

def setup_logbook(name, logdir, maxsize, extension='.txt', level=logging.INFO):
    """Setup logger that writes to file, supports multiple instances with no overlap.
       Available levels: DEBUG|INFO|WARN|ERROR
       !Legacy, usage not recommended."""
    formatter = logging.Formatter(fmt='%(asctime)s.%(msecs)03d (%(name)s) - %(message)s', datefmt='%d-%m-%y %H:%M:%S')
    date = datetime.today().strftime('%Y-%m-%d')
    log_path = str(logdir + name +'_' + date + extension)
    try:
        handler = RotatingFileHandler(log_path, maxBytes=maxsize, backupCount=1)
    except FileNotFoundError:
        os.mkdir(logdir)
        handler = RotatingFileHandler(log_path, maxBytes=maxsize, backupCount=1)
    handler.setFormatter(formatter)
    logger = logging.getLogger(name)
    logger.setLevel(level)
    logger.addHandler(handler)
    return logger


import requests
import functools

PAGESPEED_MAX_RETRIES = 3
RETRY_COUNTER = 0

def get_vtex_categories(
    customer,
    product_name,
    environment="vtexcommercestable"):
    """
    This function will get the categories related to a specific product.

    Args:
        customer (str): Customer name which has the product
        product_name (str): Product name as it comes from the ga_wrapper functions 
                            access_block, exit_rate_block and sales_block.
        environment (str): Environment to use
    
    Returns: 
        unique_categories (list): A list of the categories names.

    Notes:
        No retries implemented, so this is a candidade for data fetch loss.
    """

    product_name = product_name.replace(" ", "-").lower()

    endpoint = f"https://{customer}.{environment}.com.br/api/catalog_system/pub/products/search/{product_name}/p"

    r = requests.get(endpoint).json()
    if r and isinstance(r[0], dict):
        categories = r[0].get('categories')

        categories = list(set(functools.reduce(lambda x, y: x + y, map(
                lambda string: list(filter(
                    lambda element: element,
                    string.split('/')
                )),
                categories
            ))))

        return categories
    else:
        return ["No category"]

def get_pagespeed_required_urls(customer, environment="vtexcommercestable"):
    """
    """
    global RETRY_COUNTER
    endpoint = f"https://{customer}.{environment}.com.br/api/catalog_system/pub/products/search?fq=isAvailablePerSalesChannel_1:1&_from=1&_to=3&O=OrderByTopSaleDESC"
    productids = []
    product_text_links = []
    categories = []
    response = requests.get(endpoint)
    response_json = response.json()

    if response.status_code in range(200, 300):
        for item in response_json:
            productids.append(item.get('productId'))
            product_text_links.append(item.get('linkText'))
            categories.append(item.get('categories')[0])
            RETRY_COUNTER = 0

    while response.status_code not in range(200, 300):
        print(f"The status_code of the request is {response.status_code}")
        RETRY_COUNTER += 1
        print(f"Retry number {RETRY_COUNTER}")
        response = requests.get(endpoint)
        if RETRY_COUNTER == PAGESPEED_MAX_RETRIES:
            RETRY_COUNTER = 0
            productids = [123,123,123] # Dummy prodIDs so code doesn't break
            break

    url_checkout = f"https://{customer}.{environment}.com.br/checkout/cart/add?sku={productids[0]}&qty=1&seller=1&sku={productids[1]}&qty=1&seller=1&sku={productids[2]}&qty=1&seller=1&redirect=true&sc=1"
    return url_checkout, product_text_links, categories

if __name__ == "__main__":
    print(get_pagespeed_required_urls('fibracirurgica'))


#!/usr/bin/env python
"""Core module for mongodb related operations"""

from sallron.util import settings, parser
import pymongo
import json
import os

class MongoDB():
    def __init__(self):
        """
        Single cluster, optionally database-specific pymongo object
        with utility-focused functions
        """

        self.client = MongoDB.make_client()
        self.databases = [database for database in self.client.list_database_names() if database not in settings.ADMIN_DATABASES]

    def database(self, database_name):
        if database_name not in self.databases:
            raise("Database not found")
        else:
            return getattr(self.client, database_name)

    @staticmethod
    def make_client():
        """
        Utility funciton to load MongoClient

        Args:
            -

        Returns:
            Client for a MongoDB instance
        """

        client = pymongo.MongoClient(settings.MONGO_CONN_STR)

        return client

    def insert_data(self, database, collection, data):
        """
        Utility function to insert data to a collection

        Args:
            database (str): Name of the brand in which data will be inserted
            collection (str): Name of the collection
            data (dict): Input data dict
        """

        # clean punctuation from data
        data = parser.camelify(data)
        data = parser.remove_punctuation(data)

        collection = getattr(getattr(self.client, database), collection)
        collection.insert_one(data)

    def delete_data(self, database, collection, _filter):
        """
        Utility function to delete data from collection that matches the query

        Args:
            database (str): Name of the brand in which data will be inserted
            _filter (dict): Input filter dict
            collection (str): Name of the collection
        """

        collection = getattr(getattr(self.client, database), collection)
        collection.delete_one(_filter)

    def fetch_data(self, database, collection, _filter, _one=True):
        """
        Utility function to make a filter in any collection

        Args:
            database (str): Name of the brand in which data will be inserted
            collection (str): Name of the collection
            _filter (dict): Input filter dict
            one (bool): Indicates whether to return only one object or a batch

        Returns:
            dict: Dict containing object that matches the filter
        """

        collection = getattr(getattr(self.client, database), collection)
        _object = collection.find_one(_filter) if _one else collection.find(_filter)

        return _object


class MongoDBTesting():
    def __init__(self):
        """
        Single cluster, optionally database-specific pymongo object
        with utility-focused functions
        """

        self.client = MongoDB.make_client()
        self.databases = [database for database in self.client.list_database_names() if database not in ["admin", "local"]]

    def database(self, database_name):
        if database_name not in self.databases:
            raise("Database not found")
        else:
            return getattr(self.client, database_name)

    @staticmethod
    def make_client():
        """
        Utility funciton to load MongoClient

        Args:
            -

        Returns:
            Client for a MongoDB instance
        """

        client = pymongo.MongoClient(settings.MONGO_TESTING_CONN_STR)

        return client

    def insert_data(self, database, collection, data):
        """
        Utility function to insert data to a collection

        Args:
            database (str): Name of the brand in which data will be inserted
            collection (str): Name of the collection
            data (dict): Input data dict
        """

        # clean punctuation from data
        data = parser.camelify(data)
        data = parser.remove_punctuation(data)

        collection = getattr(getattr(self.client, database), collection)
        collection.insert_one(data)

    def delete_data(self, database, collection, _filter):
        """
        Utility function to delete data from collection that matches the query

        Args:
            database (str): Name of the brand in which data will be inserted
            _filter (dict): Input filter dict
            collection (str): Name of the collection
        """

        collection = getattr(getattr(self.client, database), collection)
        collection.delete_one(_filter)

    def fetch_data(self, database, collection, _filter, _one=True):
        """
        Utility function to make a filter in any collection

        Args:
            database (str): Name of the brand in which data will be inserted
            collection (str): Name of the collection
            _filter (dict): Input filter dict
            one (bool): Indicates whether to return only one object or a batch

        Returns:
            dict: Dict containing object that matches the filter
        """

        collection = getattr(getattr(self.client, database), collection)
        _object = collection.find_one(_filter) if _one else collection.find(_filter)

        return _object

    def insert_customer(self, kwargs):
        """
        Utility function to insert a new customer

        Kwargs:
            customer-name (str): Name of the customer
            payed-until (datetime.datetime): Period in which data will be captured for the customer
            active (bool): Indicates whether customer is active or not
            api-key (str): VTEX API key
            api-token (str): VTEX API token
            environment (str): Input domain defining the environment. Normally 'vtexcommercestable'
            account-name (str): VTEX account name, e.g. fibracirurgica, electrolux, etc
            service-account (dict): Google Analytics key dict
            view-id (str): Google Analytics view ID
            timezone (str): String defining the timezone to get data. Check
                            https://gist.github.com/matheushent/40b004923b2293721d136b02b8426f67
                            for a complete list of available timezones
            pagespeed-key (str): Input key for PageSpeed API
            url (List): Comma-separated list of URLs for requesting by PageSpeed
        """

        collection = self.client[kwargs.get("customer-name")]['info']

        collection.insert_one(kwargs)

class MongoAdmin():
    def __init__(self):
        """
        Single cluster, optionally database-specific pymongo object
        with utility-focused functions
        """

        self.client = self.make_client()

        self.databases = self.client.list_database_names()[2:]

    def make_client(self):
        """
        Utility funciton to load MongoClient

        Args:
            -

        Returns:
            Client for a MongoDB instance
        """
    
        client = pymongo.MongoClient(settings.MONGO_CONN_STR)

        return client

    def capped_collection(self, coll_name, db, max_size=50000000):
        """
        Utility function to create a capped collection

        Args:
            coll_name (str): Name of the collection
            db (pymongo.database.Database): Database instance in which the collection will be created
            max_size (int): Maximum size allowed to the collection
        """

        db.create_collection(coll_name, capped=True, size=max_size)

    def insert_data(self, database, collection, data):
        """
        Utility function to insert data to a collection

        Args:
            database (str): Name of the brand in which data will be inserted
            collection (str): Name of the collection
            data (dict): Input data dict
        """

        collection = self.client[database][collection]
        collection.insert_one(data)

    def delete_data(self, database, _filter, collection='data'):
        """
        Utility function to delete data from collection that matches the query

        Args:
            database (str): Name of the brand in which data will be inserted
            _filter (dict): Input filter dict
            collection (str): Name of the collection
        """

        collection = self.client.get(database)[collection]
        collection.delete_one(_filter)

    def _filter(self, database, _filter, collection='data', one=True):
        """
        Utility function to make a filter in any collection

        Args:
            database (str): Name of the brand in which data will be inserted
            collection (str): Name of the collection
            _filter (dict): Input filter dict
            one (bool): Indicates whether to return only one object or a batch

        Returns:
            dict: Dict containing object that matches the filter
        """

        collection = self.client.get(database)[collection]

        if one:
            _object = collection.find_one(_filter)
        else:
            _object = collection.find(_filter)

        return _object

    def insert_customer(self, kwargs):
        """
        Utility function to insert a new customer

        Kwargs:
            customer-name (str): Name of the customer
            payed-until (datetime.datetime): Period in which data will be captured for the customer
            active (bool): Indicates whether customer is active or not
            api-key (str): VTEX API key
            api-token (str): VTEX API token
            environment (str): Input domain defining the environment. Normally 'vtexcommercestable'
            account-name (str): VTEX account name, e.g. fibracirurgica, electrolux, etc
            service-account (dict): Google Analytics key dict
            view-id (str): Google Analytics view ID
            timezone (str): String defining the timezone to get data. Check
                            https://gist.github.com/matheushent/40b004923b2293721d136b02b8426f67
                            for a complete list of available timezones
            pagespeed-key (str): Input key for PageSpeed API
            url (List): Comma-separated list of URLs for requesting by PageSpeed
        """

        collection = self.client[kwargs.get("customer-name")].info

        collection.insert_one(kwargs)

    def delete_customer(self, name):
        """
        Utility function to delete a customer.

        Args:
            name (str): Name of the customer
        """

        self.client.drop_database(name)

    def activate_customer(self, name):
        """
        Utility function to activate the status of a customer

        Args:
            name (str): Name of the customer
        """

        collection = self.client[name].info

        collection.update_one(
            {
                "customer-name": name
            },
            {
                "$set": {
                    "active": True
                }
            }
        )

    def deactivate_customer(self, name):
        """
        Utility function to deactivate the status of a customer

        Args:
            name (str): Name of the customer
        """

        collection = self.client[name].info

        collection.update_one(
            {
                "customer-name": name
            },
            {
                "$set": {
                    "active": False
                }
            }
        )


from sallron.util import settings, logger, executor
from sallron.enforcer import write_pid
from sallron.db import caching, mongo
import datetime
import schedule
import logging
import inspect
import pytz
import os

class TheEyeofSauron:
    def __init__(self, customer, info, interface_class, interface_name):
        write_pid(str(os.getpid()))
        # Logging
        self.sauron_logger = logging.getLogger('sauron')
        self.sauron_logger.info('Intializing.')

        # Internal class variables
        self._customer = customer
        self._info = info
        self._interface = interface_class(**info.get(interface_name))
        self._interface_name = interface_name

    def setup_schedulers(self, dictator, mongo_client):

        # filter methods and get only the blocks
        methods = filter(
            lambda method: not isinstance(
                inspect.getattr_static(self._interface, method.__name__),
                staticmethod
            ),
            filter(
                lambda method: not method.__name__.startswith('_'),
                filter(
                    inspect.ismethod,
                    map(
                        lambda name: getattr(self._interface, name),
                        dir(self._interface)
                    )
                )
            )
        )

        for method in methods:
            if method.__name__ != "load_time_block":
                schedule.every(dictator.get(self._interface_name).get(method.__name__)).minutes.do(
                    self.fetch_and_store,
                    method=method,
                    period=dictator.get(self._interface_name).get(method.__name__),
                    mongo_client=mongo_client
                )
            else:
                schedule.every().day.at(
                    datetime.time(
                        hour=23, 
                        minute=00, 
                        tzinfo=pytz.timezone(settings.TIMEZONE)).strftime("%H:%M")
                        ).do(
                            self.fetch_and_store,
                            method=method,
                            period=dictator.get(self._interface_name).get(method.__name__),
                            mongo_client=mongo_client)

    def fetch_and_store(self, method, period, mongo_client):
        self.sauron_logger.info(
            'Fetching and storing: {0}'.format(
                method.__name__
            ))

        fetched_data = method(period=period)

        self.sauron_logger.debug('Data received: {0}'.format(fetched_data))

        mongo_client.insert_data(self._customer, method.__name__, fetched_data)

def ring_ruler(interface, interface_name, log_dir, test=False):
    mongo_client = mongo.MongoDB()

    def fn(_customer, _interface, _interface_name):

        mongo_client = mongo.MongoDB()

        info = mongo_client.fetch_data(_customer, settings.ADMIN_COLLECTION, {})

        saruman = TheEyeofSauron(_customer, info, _interface, _interface_name)

        saruman.setup_schedulers(info.get("schedulers"), mongo_client)

    for customer in mongo_client.databases:
        fn(customer, interface, interface_name)

    # Caching commented out since executor will already reset the process daily
    # cache_server = caching.RedisCache()
    # cache_server.schedule_new_customers_checking(mongo_client, interface, interface_name, fn)

    schedule.run_all() # run all jobs straight away so we're not left without data for long

    executor.schedule_log_sending_and_kill_process(log_dir, settings.TIMEZONE)
    if not(test):
        while True:
            schedule.run_pending()

def configureye(**kwargs):
    for key, value in kwargs.items():
        setattr(settings, key, value)


from sallron.util import settings
from datetime import datetime
import psutil
import os

def program_settings():
    pid = os.getpid()
    return {
        'Timestamp' : str(datetime.now()),
        'OS' : settings.OS,
        'Interface' : settings.INTERFACE_NAME,
        'Timezone' : settings.TIMEZONE,
        'AWSRegion' : settings.AWS_REGION,
        'PID' : pid,
        'SID' : os.getsid(pid)
    }

def system_stats():
    uname_result = os.uname()
    return {
            'CPUt' : dict(psutil.cpu_times()._asdict()),
            'CPU%' : psutil.cpu_percent(),
            'VMem%' : dict(psutil.virtual_memory()._asdict()).get('percent'),
            'SMem%' : dict(psutil.swap_memory()._asdict()).get('percent'),
            'Disk%' : dict(psutil.disk_usage('/')._asdict()).get('percent'),
            'Sysname' : uname_result.sysname,
            'Nodename' : uname_result.nodename,
            'Sysrelease' : uname_result.release,
            'Sysversion' : uname_result.version.split(' ')[-1],
            'Sysmachine' : uname_result.machine
    }

def metadata_dict():
    return {**program_settings(), **system_stats()}



"""Core module for AWS S3 related operations"""
from sallron.util import settings
from functools import wraps
import pickle
import boto3
import os

def _aws_auth(f):

    @wraps(f)
    def decorator(*args, **kwargs):

        if not (settings.AWS_ACCESS_KEY_ID and settings.AWS_SECRET_ACCESS_KEY_ID and settings.AWS_REGION and settings.LOGGING_BUCKET):
            auth = False
        else:
            auth = True

        return f(auth, *args, **kwargs)
    
    return decorator

@_aws_auth
def get_bucket(aws_auth):
    if aws_auth:
        s3 = boto3.session.Session(
            aws_access_key_id=settings.AWS_ACCESS_KEY_ID,
            aws_secret_access_key=settings.AWS_SECRET_ACCESS_KEY_ID
        ).resource("s3", region_name=settings.AWS_REGION)
        return s3.Bucket(settings.LOGGING_BUCKET)

    else:
        print('AWS credentials not set.')
        return None

def send_to_s3(obj_paths):
    """
    Utility function to send objects to S3

    Args:
        aws_auth (bool): Boolean indicating whether AWS settings were set or not
        obj_path (str): Path to object to be sent
    """

    bucket = get_bucket()

    for obj_path in obj_paths:
        # ensure everything needed is set
        if os.path.exists(obj_path):

            try:
                # note it expects a obj_path following path/to/obj.txt format!
                # gets the last name of this sequence
                bucket.upload_file(obj_path, obj_path.split('/')[-1])
                # remove the current log file
                os.remove(obj_path)
                print('Files successfully sent to S3.')
            except Exception as e:
                print(e)
                print(obj_path)
                pass

        else:
            print('File path not found.')
            pass


#!/usr/bin/env python
"""Core module for parsing related operations"""

import re

PUNCTUATION = '!"#$%&\'()*+,./:;<=>?@[\\]^_`{|}~'

def remove_punctuation(obj):
    """
    Utility funciton to remove all punctuation from a obj

    e.g.:
    obj = {
        "sampling_level": [
            "field.vision": 23,
            "field.name": "Leo_di_Caprio"
        ]
    }

    => {
        "sampling_level": [
            "fieldvision": 23,
            "fieldname": "Leo_di_Caprio"
        ]
    }

    Args:
        obj (dict): Dict object containing the query for GA

    Returns:
        function: Recursive function to modify the object
    """

    def func(string): return ''.join(
        list(filter(
            lambda char: char not in PUNCTUATION,
            string
        ))
    )

    return modify_object(obj, func)

def camelify(obj):
    """
    Utility function to modify a string removing the underscores

    e.g.:
    obj = {
        "sampling_level": [
            "field_vision": 23,
            "field_name": "Leo_di_Caprio"
        ]
    }

    => {
        "samplingLevel": [
            "fieldVision": 23,
            "fieldName": "LeoDiCaprio"
        ]
    }

    Args:
        obj (dict): Dict object containing the query for GA

    Returns:
        function: Recursive function to modify the object
    """

    def func(string): return re.sub(
        r'_(\w)', lambda match: match.group(1).upper(), string)

    return modify_object(obj, func)

def decamelify(obj):
    """
    Opposite of camelify
    """

    def func(string): return re.sub(
        '([a-z0-9])([A-Z])', r'\1_\2', string).lower()

    return modify_object(obj, func)


def modify_object(obj, func):
    """
    Recursive function to go through indented objects and modify it

    Args:
        obj (dict, list, bool, str): Value of key-value pair of query object from GA
        func (function): Input function to modify obj

    Returns:
        obj: Modified object
    """

    if isinstance(obj, dict):

        temp_obj = {}
        for key in obj:
            temp_obj[func(key)] = modify_object(obj[key], func)

        return temp_obj

    if isinstance(obj, list):

        temp_obj = []
        for key in obj:
            temp_obj.append(modify_object(key, func))

        return temp_obj

    if isinstance(obj, bool):
        return obj

    if isinstance(obj, str) and not obj.isupper():
        return func(obj)

    return obj


#!/usr/bin/env python

# GENERAL CONFIG

INTERFACE_NAME = 'interface_name'

TIMEZONE = "America/Sao_Paulo" # timezone for restarting and acquiring load_time_block 

OS = 'UBUNTU'
# OS = 'MAC'

DISCORD_WEBHOOK = "https://discord.com/api/webhooks/123..."

# LOGGER CONFIG

SAVE_LOGS = False

LOG_DIR = "logs/"

LOG_LEVEL='INFO'

MAX_LOG_SIZE = 100000000 # 100Mb

MAX_LOG_BACKUPS = 5

# MONGO CONFIG

MONGO_CONN_STR = "mongodb+srv://..."

ADMIN_COLLECTION = "info" # collection containing credentials and function parameters

ADMIN_DATABASES = ["admin", "local", "temporary-tokens"] # databases to ignore when fetching customers

# AWS CONFIG

AWS_ACCESS_KEY_ID = "ABC..."

AWS_SECRET_ACCESS_KEY_ID = "cD3..."

AWS_REGION = "us-east-2"

LOGGING_BUCKET = "monitor.logging"


from sallron.util import s3, logger, discord, settings
import schedule
import datetime
import pytz
import sys

def send_to_s3_and_kill(log_dir):
    """
    Utility function to send the log to s3 and kill the process

    Args:
        log_dir (str): Directory containing log to be sent
    """
    masterlog_path = logger.get_logname(log_dir, 'master')
    errorslog_path = logger.get_logname(log_dir, 'errors')

    discord.send_message(
        message = f'Bucket hit: {settings.LOGGIN_BUCKET}',
        level = 'default',
        title = 'Logs backed up to S3 bucket',
        description = f'Files sent: \n{masterlog_path}\n{errorslog_path}'
    )

    s3.send_to_s3([masterlog_path, errorslog_path])

    sys.exit("Exiting from process after send log to S3.")

def schedule_log_sending_and_kill_process(log_dir, timezone):
    """
    Utility function to schedule the sending of log file to S3 and kill the hole process
    Args:
        obj_path (str): Path to object to be sent
        timezone (str): Timezone string supported by pytz
    """

    schedule.every().day.at(
        datetime.time(hour=0, minute=1, tzinfo=pytz.timezone(timezone)).strftime("%H:%M")
    ).do(send_to_s3_and_kill, log_dir=log_dir)